@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2016 Daniel Schadt
+# Copyright 2015-2017 Daniel Schadt
 #
 # This file is part of qutebrowser.
 #
@@ -25,7 +25,6 @@ import io
 import os
 import re
 import sys
-import collections
 import uuid
 import email.policy
 import email.generator
@@ -34,15 +33,21 @@ import email.mime.multipart
 import email.message
 import quopri
 
+import attr
 from PyQt5.QtCore import QUrl
 
 from qutebrowser.browser import downloads
 from qutebrowser.browser.webkit import webkitelem
 from qutebrowser.utils import log, objreg, message, usertypes, utils, urlutils
 
-_File = collections.namedtuple('_File',
-                               ['content', 'content_type', 'content_location',
-                                'transfer_encoding'])
+
+@attr.s
+class _File:
+
+    content = attr.ib()
+    content_type = attr.ib()
+    content_location = attr.ib()
+    transfer_encoding = attr.ib()
 
 
 _CSS_URL_PATTERNS = [re.compile(x) for x in [
@@ -174,7 +179,7 @@ class MHTMLWriter:
         root_content: The root content as bytes.
         content_location: The url of the page as str.
         content_type: The MIME-type of the root content as str.
-        _files: Mapping of location->_File namedtuple.
+        _files: Mapping of location->_File object.
     """
 
     def __init__(self, root_content, content_location, content_type):
@@ -242,7 +247,7 @@ class _Downloader:
 
     Attributes:
         tab: The AbstractTab which contains the website that will be saved.
-        dest: Destination filename.
+        target: DownloadTarget where the file should be downloaded to.
         writer: The MHTMLWriter object which is used to save the page.
         loaded_urls: A set of QUrls of finished asset downloads.
         pending_downloads: A set of unfinished (url, DownloadItem) tuples.
@@ -252,9 +257,9 @@ class _Downloader:
         _win_id: The window this downloader belongs to.
     """
 
-    def __init__(self, tab, dest):
+    def __init__(self, tab, target):
         self.tab = tab
-        self.dest = dest
+        self.target = target
         self.writer = None
         self.loaded_urls = {tab.url()}
         self.pending_downloads = set()
@@ -349,8 +354,8 @@ class _Downloader:
 
         # Using the download manager to download host-blocked urls might crash
         # qute, see the comments/discussion on
-        # https://github.com/The-Compiler/qutebrowser/pull/962#discussion_r40256987
-        # and https://github.com/The-Compiler/qutebrowser/issues/1053
+        # https://github.com/qutebrowser/qutebrowser/pull/962#discussion_r40256987
+        # and https://github.com/qutebrowser/qutebrowser/issues/1053
         host_blocker = objreg.get('host-blocker')
         if host_blocker.is_blocked(url):
             log.downloads.debug("Skipping {}, host-blocked".format(url))
@@ -462,14 +467,34 @@ class _Downloader:
             return
         self._finished_file = True
         log.downloads.debug("All assets downloaded, ready to finish off!")
+
+        if isinstance(self.target, downloads.FileDownloadTarget):
+            fobj = open(self.target.filename, 'wb')
+        elif isinstance(self.target, downloads.FileObjDownloadTarget):
+            fobj = self.target.fileobj
+        elif isinstance(self.target, downloads.OpenFileDownloadTarget):
+            try:
+                fobj = downloads.temp_download_manager.get_tmpfile(
+                    self.tab.title() + '.mhtml')
+            except OSError as exc:
+                msg = "Download error: {}".format(exc)
+                message.error(msg)
+                return
+        else:
+            raise ValueError("Invalid DownloadTarget given: {!r}"
+                             .format(self.target))
+
         try:
-            with open(self.dest, 'wb') as file_output:
-                self.writer.write_to(file_output)
+            with fobj:
+                self.writer.write_to(fobj)
         except OSError as error:
             message.error("Could not save file: {}".format(error))
             return
         log.downloads.debug("File successfully written.")
-        message.info("Page saved as {}".format(self.dest))
+        message.info("Page saved as {}".format(self.target))
+
+        if isinstance(self.target, downloads.OpenFileDownloadTarget):
+            utils.open_file(fobj.name, self.target.cmdline)
 
     def _collect_zombies(self):
         """Collect done downloads and add their data to the MHTML file.
@@ -501,34 +526,37 @@ class _NoCloseBytesIO(io.BytesIO):
         super().close()
 
 
-def _start_download(dest, tab):
+def _start_download(target, tab):
     """Start downloading the current page and all assets to an MHTML file.
 
     This will overwrite dest if it already exists.
 
     Args:
-        dest: The filename where the resulting file should be saved.
+        target: The DownloadTarget where the resulting file should be saved.
         tab: Specify the tab whose page should be loaded.
     """
-    loader = _Downloader(tab, dest)
+    loader = _Downloader(tab, target)
     loader.run()
 
 
-def start_download_checked(dest, tab):
+def start_download_checked(target, tab):
     """First check if dest is already a file, then start the download.
 
     Args:
-        dest: The filename where the resulting file should be saved.
+        target: The DownloadTarget where the resulting file should be saved.
         tab: Specify the tab whose page should be loaded.
     """
-    # The default name is 'page title.mht'
+    if not isinstance(target, downloads.FileDownloadTarget):
+        _start_download(target, tab)
+        return
+    # The default name is 'page title.mhtml'
     title = tab.title()
-    default_name = utils.sanitize_filename(title + '.mht')
+    default_name = utils.sanitize_filename(title + '.mhtml')
 
     # Remove characters which cannot be expressed in the file system encoding
     encoding = sys.getfilesystemencoding()
     default_name = utils.force_encoding(default_name, encoding)
-    dest = utils.force_encoding(dest, encoding)
+    dest = utils.force_encoding(target.filename, encoding)
 
     dest = os.path.expanduser(dest)
 
@@ -549,8 +577,9 @@ def start_download_checked(dest, tab):
         message.error("Directory {} does not exist.".format(folder))
         return
 
+    target = downloads.FileDownloadTarget(path)
     if not os.path.isfile(path):
-        _start_download(path, tab=tab)
+        _start_download(target, tab=tab)
         return
 
     q = usertypes.Question()
@@ -560,5 +589,5 @@ def start_download_checked(dest, tab):
         html.escape(path))
     q.completed.connect(q.deleteLater)
     q.answered_yes.connect(functools.partial(
-        _start_download, path, tab=tab))
+        _start_download, target, tab=tab))
     message.global_bridge.ask(q, blocking=False)
