@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2016 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,22 +19,31 @@
 
 """Module containing command managers (SearchRunner and CommandRunner)."""
 
-import collections
 import traceback
 import re
 
+import attr
 from PyQt5.QtCore import pyqtSlot, QUrl, QObject
 
-from qutebrowser.config import config, configexc
+from qutebrowser.config import config
 from qutebrowser.commands import cmdexc, cmdutils
 from qutebrowser.utils import message, objreg, qtutils, usertypes, utils
 from qutebrowser.misc import split
 
 
-ParseResult = collections.namedtuple('ParseResult', ['cmd', 'args', 'cmdline'])
 last_command = {}
 
 
+@attr.s
+class ParseResult:
+
+    """The result of parsing a commandline."""
+
+    cmd = attr.ib()
+    args = attr.ib()
+    cmdline = attr.ib()
+
+
 def _current_url(tabbed_browser):
     """Convenience method to get the current url."""
     try:
@@ -81,19 +90,17 @@ def replace_variables(win_id, arglist):
     return args
 
 
-class CommandRunner(QObject):
+class CommandParser:
 
-    """Parse and run qutebrowser commandline commands.
+    """Parse qutebrowser commandline commands.
 
     Attributes:
-        _win_id: The window this CommandRunner is associated with.
+
         _partial_match: Whether to allow partial command matches.
     """
 
-    def __init__(self, win_id, partial_match=False, parent=None):
-        super().__init__(parent)
+    def __init__(self, partial_match=False):
         self._partial_match = partial_match
-        self._win_id = win_id
 
     def _get_alias(self, text, default=None):
         """Get an alias from the config.
@@ -108,9 +115,10 @@ class CommandRunner(QObject):
         """
         parts = text.strip().split(maxsplit=1)
         try:
-            alias = config.get('aliases', parts[0])
-        except (configexc.NoOptionError, configexc.NoSectionError):
+            alias = config.val.aliases[parts[0]]
+        except KeyError:
             return default
+
         try:
             new_cmd = '{} {}'.format(alias, parts[1])
         except IndexError:
@@ -119,7 +127,7 @@ class CommandRunner(QObject):
             new_cmd += ' '
         return new_cmd
 
-    def parse_all(self, text, aliases=True, *args, **kwargs):
+    def _parse_all_gen(self, text, aliases=True, *args, **kwargs):
         """Split a command on ;; and parse all parts.
 
         If the first command in the commandline is a non-split one, it only
@@ -133,7 +141,8 @@ class CommandRunner(QObject):
         Yields:
             ParseResult tuples.
         """
-        if not text.strip():
+        text = text.strip().lstrip(':').strip()
+        if not text:
             raise cmdexc.NoSuchCommandError("No command given")
 
         if aliases:
@@ -153,6 +162,10 @@ class CommandRunner(QObject):
         for sub in sub_texts:
             yield self.parse(sub, *args, **kwargs)
 
+    def parse_all(self, *args, **kwargs):
+        """Wrapper over parse_all."""
+        return list(self._parse_all_gen(*args, **kwargs))
+
     def parse(self, text, *, fallback=False, keep=False):
         """Split the commandline text into command and arguments.
 
@@ -252,6 +265,20 @@ class CommandRunner(QObject):
             # already.
             return split_args
 
+
+class CommandRunner(QObject):
+
+    """Parse and run qutebrowser commandline commands.
+
+    Attributes:
+        _win_id: The window this CommandRunner is associated with.
+    """
+
+    def __init__(self, win_id, partial_match=False, parent=None):
+        super().__init__(parent)
+        self._parser = CommandParser(partial_match=partial_match)
+        self._win_id = win_id
+
     def run(self, text, count=None):
         """Parse a command from a line of text and run it.
 
@@ -266,7 +293,7 @@ class CommandRunner(QObject):
                                   window=self._win_id)
         cur_mode = mode_manager.mode
 
-        for result in self.parse_all(text):
+        for result in self._parser.parse_all(text):
             if result.cmd.no_replace_variables:
                 args = result.args
             else:
@@ -293,17 +320,5 @@ class CommandRunner(QObject):
         """Run a command and display exceptions in the statusbar."""
         try:
             self.run(text, count)
-        except (cmdexc.CommandMetaError, cmdexc.CommandError) as e:
-            message.error(str(e), stack=traceback.format_exc())
-
-    @pyqtSlot(str, int)
-    def run_safely_init(self, text, count=None):
-        """Run a command and display exceptions in the statusbar.
-
-        Contrary to run_safely, error messages are queued so this is more
-        suitable to use while initializing.
-        """
-        try:
-            self.run(text, count)
-        except (cmdexc.CommandMetaError, cmdexc.CommandError) as e:
+        except cmdexc.Error as e:
             message.error(str(e), stack=traceback.format_exc())
