@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2016 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -25,7 +25,7 @@ import tempfile
 
 from PyQt5.QtCore import pyqtSignal, pyqtSlot, QObject, QSocketNotifier
 
-from qutebrowser.utils import message, log, objreg, standarddir
+from qutebrowser.utils import message, log, objreg, standarddir, utils
 from qutebrowser.commands import runners
 from qutebrowser.config import config
 from qutebrowser.misc import guiprocess
@@ -64,14 +64,19 @@ class _QtFIFOReader(QObject):
     def read_line(self):
         """(Try to) read a line from the FIFO."""
         log.procs.debug("QSocketNotifier triggered!")
-        self._notifier.setEnabled(False)
         try:
-            for line in self._fifo:
-                self.got_line.emit(line.rstrip('\r\n'))
-                self._notifier.setEnabled(True)
-        except UnicodeDecodeError as e:
-            log.misc.error("Invalid unicode in userscript output: {}"
-                           .format(e))
+            self._notifier.setEnabled(False)
+            try:
+                for line in self._fifo:
+                    self.got_line.emit(line.rstrip('\r\n'))
+                    self._notifier.setEnabled(True)
+            except UnicodeDecodeError as e:
+                log.misc.error("Invalid unicode in userscript output: {}"
+                               .format(e))
+        except RuntimeError as e:
+            # For unknown reasons, read_line can still get called after the
+            # QSocketNotifier was already deleted...
+            log.procs.debug("While reading userscript output: {}".format(e))
 
     def cleanup(self):
         """Clean up so the FIFO can be closed."""
@@ -371,7 +376,7 @@ def _lookup_path(cmd):
     """
     directories = [
         os.path.join(standarddir.data(), "userscripts"),
-        os.path.join(standarddir.system_data(), "userscripts"),
+        os.path.join(standarddir.data(system=True), "userscripts"),
     ]
     for directory in directories:
         cmd_path = os.path.join(directory, cmd)
@@ -401,9 +406,9 @@ def run_async(tab, cmd, *args, win_id, env, verbose=False):
                                 window=win_id)
     commandrunner = runners.CommandRunner(win_id, parent=tabbed_browser)
 
-    if os.name == 'posix':
+    if utils.is_posix:
         runner = _POSIXUserscriptRunner(tabbed_browser)
-    elif os.name == 'nt':  # pragma: no cover
+    elif utils.is_windows:  # pragma: no cover
         runner = _WindowsUserscriptRunner(tabbed_browser)
     else:  # pragma: no cover
         raise UnsupportedError
@@ -412,7 +417,7 @@ def run_async(tab, cmd, *args, win_id, env, verbose=False):
         lambda cmd:
         log.commands.debug("Got userscript command: {}".format(cmd)))
     runner.got_cmd.connect(commandrunner.run_safely)
-    user_agent = config.get('network', 'user-agent')
+    user_agent = config.val.content.headers.user_agent
     if user_agent is not None:
         env['QUTE_USER_AGENT'] = user_agent
 
