@@ -23,6 +23,8 @@ import os
 import sys
 import functools
 import html
+import ctypes
+import ctypes.util
 
 import attr
 from PyQt5.QtCore import Qt
@@ -31,7 +33,7 @@ from PyQt5.QtWidgets import (QApplication, QDialog, QPushButton, QHBoxLayout,
 from PyQt5.QtNetwork import QSslSocket
 
 from qutebrowser.config import config
-from qutebrowser.utils import usertypes, objreg, version, qtutils, log
+from qutebrowser.utils import usertypes, objreg, version, qtutils, log, utils
 from qutebrowser.misc import objects, msgbox
 
 
@@ -154,6 +156,19 @@ def _show_dialog(*args, **kwargs):
     sys.exit(usertypes.Exit.err_init)
 
 
+def _nvidia_shader_workaround():
+    """Work around QOpenGLShaderProgram issues.
+
+    NOTE: This needs to be called before _handle_nouveau_graphics, or some
+    setups will segfault in version.opengl_vendor().
+
+    See https://bugs.launchpad.net/ubuntu/+source/python-qt4/+bug/941826
+    """
+    assert objects.backend == usertypes.Backend.QtWebEngine, objects.backend
+    if utils.is_linux:
+        ctypes.CDLL(ctypes.util.find_library("GL"), mode=ctypes.RTLD_GLOBAL)
+
+
 def _handle_nouveau_graphics():
     assert objects.backend == usertypes.Backend.QtWebEngine, objects.backend
 
@@ -352,6 +367,7 @@ def init():
     if objects.backend == usertypes.Backend.QtWebEngine:
         _handle_ssl_support()
         _handle_wayland()
+        _nvidia_shader_workaround()
         _handle_nouveau_graphics()
     else:
         assert objects.backend == usertypes.Backend.QtWebKit, objects.backend
