@@ -24,7 +24,8 @@ from PyQt5.QtWidgets import (QLineEdit, QWidget, QHBoxLayout, QLabel,
                              QStyleOption, QStyle, QLayout, QApplication)
 from PyQt5.QtGui import QValidator, QPainter
 
-from qutebrowser.utils import utils, objreg, qtutils, log, usertypes
+from qutebrowser.config import config
+from qutebrowser.utils import utils, qtutils, log, usertypes
 from qutebrowser.misc import cmdhistory, objects
 
 
@@ -245,8 +246,7 @@ class WrapperLayout(QLayout):
     def sizeHint(self):
         return self._widget.sizeHint()
 
-    def itemAt(self, _index):  # pragma: no cover
-        # For some reason this sometimes gets called by Qt.
+    def itemAt(self, _index):
         return None
 
     def takeAt(self, _index):
@@ -260,7 +260,7 @@ class WrapperLayout(QLayout):
         self._widget = widget
         container.setFocusProxy(widget)
         widget.setParent(container)
-        if (qtutils.version_check('5.8.0', exact=True) and
+        if (qtutils.version_check('5.8.0', exact=True, compiled=False) and
                 objects.backend == usertypes.Backend.QtWebEngine and
                 container.window() and
                 container.window().windowHandle() and
@@ -289,8 +289,7 @@ class FullscreenNotification(QLabel):
             padding: 30px;
         """)
 
-        key_config = objreg.get('key-config')
-        all_bindings = key_config.get_reverse_bindings_for('normal')
+        all_bindings = config.key_instance.get_reverse_bindings_for('normal')
         bindings = all_bindings.get('fullscreen --leave')
         if bindings:
             key = bindings[0]
