@@ -23,18 +23,16 @@ import os
 import os.path
 
 import sip
-from PyQt5.QtCore import pyqtSignal, QUrl, QObject, QPoint, QTimer
+from PyQt5.QtCore import QUrl, QObject, QPoint, QTimer
 from PyQt5.QtWidgets import QApplication
 import yaml
-try:
-    from yaml import CSafeLoader as YamlLoader, CSafeDumper as YamlDumper
-except ImportError:  # pragma: no cover
-    from yaml import SafeLoader as YamlLoader, SafeDumper as YamlDumper
 
-from qutebrowser.utils import (standarddir, objreg, qtutils, log, usertypes,
-                               message, utils)
+from qutebrowser.utils import (standarddir, objreg, qtutils, log, message,
+                               utils)
 from qutebrowser.commands import cmdexc, cmdutils
-from qutebrowser.config import config
+from qutebrowser.config import config, configfiles
+from qutebrowser.completion.models import miscmodels
+from qutebrowser.mainwindow import mainwindow
 
 
 default = object()  # Sentinel value
@@ -105,14 +103,8 @@ class SessionManager(QObject):
                               closed.
         _current: The name of the currently loaded session, or None.
         did_load: Set when a session was loaded.
-
-    Signals:
-        update_completion: Emitted when the session completion should get
-                           updated.
     """
 
-    update_completion = pyqtSignal()
-
     def __init__(self, base_path, parent=None):
         super().__init__(parent)
         self._current = None
@@ -259,7 +251,7 @@ class SessionManager(QObject):
                   object.
         """
         if name is default:
-            name = config.get('general', 'session-default-name')
+            name = config.val.session_default_name
             if name is None:
                 if self._current is not None:
                     name = self._current
@@ -298,15 +290,12 @@ class SessionManager(QObject):
         log.sessions.vdebug("Saving data: {}".format(data))
         try:
             with qtutils.savefile_open(path) as f:
-                yaml.dump(data, f, Dumper=YamlDumper, default_flow_style=False,
-                          encoding='utf-8', allow_unicode=True)
+                utils.yaml_dump(data, f)
         except (OSError, UnicodeEncodeError, yaml.YAMLError) as e:
             raise SessionError(e)
-        else:
-            self.update_completion.emit()
+
         if load_next_time:
-            state_config = objreg.get('state-config')
-            state_config['general']['session'] = name
+            configfiles.state['general']['session'] = name
         return name
 
     def save_autosave(self):
@@ -383,11 +372,10 @@ class SessionManager(QObject):
             name: The name of the session to load.
             temp: If given, don't set the current session.
         """
-        from qutebrowser.mainwindow import mainwindow
         path = self._get_session_path(name, check_exists=True)
         try:
             with open(path, encoding='utf-8') as f:
-                data = yaml.load(f, Loader=YamlLoader)
+                data = utils.yaml_load(f)
         except (OSError, UnicodeDecodeError, yaml.YAMLError) as e:
             raise SessionError(e)
 
@@ -405,8 +393,7 @@ class SessionManager(QObject):
                 if tab.get('active', False):
                     tab_to_focus = i
                 if new_tab.data.pinned:
-                    tabbed_browser.set_tab_pinned(
-                        new_tab, new_tab.data.pinned, loading=True)
+                    tabbed_browser.set_tab_pinned(new_tab, new_tab.data.pinned)
             if tab_to_focus is not None:
                 tabbed_browser.setCurrentIndex(tab_to_focus)
             if win.get('active', False):
@@ -424,7 +411,6 @@ class SessionManager(QObject):
             os.remove(path)
         except OSError as e:
             raise SessionError(e)
-        self.update_completion.emit()
 
     def list_sessions(self):
         """Get a list of all session names."""
@@ -433,10 +419,10 @@ class SessionManager(QObject):
             base, ext = os.path.splitext(filename)
             if ext == '.yml':
                 sessions.append(base)
-        return sessions
+        return sorted(sessions)
 
     @cmdutils.register(instance='session-manager')
-    @cmdutils.argument('name', completion=usertypes.Completion.sessions)
+    @cmdutils.argument('name', completion=miscmodels.session)
     def session_load(self, name, clear=False, temp=False, force=False):
         """Load a session.
 
@@ -463,8 +449,8 @@ class SessionManager(QObject):
                 for win in old_windows:
                     win.close()
 
-    @cmdutils.register(name=['session-save', 'w'], instance='session-manager')
-    @cmdutils.argument('name', completion=usertypes.Completion.sessions)
+    @cmdutils.register(instance='session-manager')
+    @cmdutils.argument('name', completion=miscmodels.session)
     @cmdutils.argument('win_id', win_id=True)
     @cmdutils.argument('with_private', flag='p')
     def session_save(self, name: str = default, current=False, quiet=False,
@@ -474,7 +460,7 @@ class SessionManager(QObject):
 
         Args:
             name: The name of the session. If not given, the session configured
-                  in general -> session-default-name is saved.
+                  in session_default_name is saved.
             current: Save the current session instead of the default.
             quiet: Don't show confirmation message.
             force: Force saving internal sessions (starting with an underline).
@@ -492,7 +478,7 @@ class SessionManager(QObject):
         try:
             if only_active_window:
                 name = self.save(name, only_window=win_id,
-                                 with_private=with_private)
+                                 with_private=True)
             else:
                 name = self.save(name, with_private=with_private)
         except SessionError as e:
@@ -503,7 +489,7 @@ class SessionManager(QObject):
                 message.info("Saved session {}.".format(name))
 
     @cmdutils.register(instance='session-manager')
-    @cmdutils.argument('name', completion=usertypes.Completion.sessions)
+    @cmdutils.argument('name', completion=miscmodels.session)
     def session_delete(self, name, force=False):
         """Delete a session.
 
