@@ -327,7 +327,8 @@ def change_console_formatter(level):
     console_handler.setFormatter(console_formatter)
 
 
-def qt_message_handler(msg_type, context, msg):
+def qt_message_handler(msg_type, context,  # pylint: disable=too-many-branches
+                       msg):
     """Qt message handler to redirect qWarning etc. to the logging system.
 
     Args:
@@ -421,6 +422,9 @@ def qt_message_handler(msg_type, context, msg):
                 'with: -9805',  # flake8: disable=E131
         ]
 
+    if not msg:
+        msg = "Logged empty message!"
+
     if any(msg.strip().startswith(pattern) for pattern in suppressed_msgs):
         level = logging.DEBUG
     else:
